@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2017  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2017-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -32,6 +32,7 @@
 #include "command.h"
 #include "dbus-proxy.h"
 #include "display.h"
+#include "properties.h"
 #include "network.h"
 
 struct known_network {
@@ -40,6 +41,7 @@ struct known_network {
 	char *type;
 	char *last_connected;
 	bool hidden;
+	bool autoconnect;
 };
 
 static const char *format_iso8601(const char *time_str, const char *format)
@@ -78,6 +80,13 @@ static void update_name(void *data, struct l_dbus_message_iter *variant)
 	network->name = l_strdup(value);
 }
 
+static const char *get_name(const void *data)
+{
+	const struct known_network *network = data;
+
+	return network->name;
+}
+
 static void update_type(void *data, struct l_dbus_message_iter *variant)
 {
 	struct known_network *network = data;
@@ -131,11 +140,35 @@ static const char *get_hidden_tostr(const void *data)
 	return network->hidden ? "yes" : "";
 }
 
+static void update_autoconnect(void *data, struct l_dbus_message_iter *variant)
+{
+	struct known_network *network = data;
+	bool value;
+
+	if (!l_dbus_message_iter_get_variant(variant, "b", &value)) {
+		network->autoconnect = false;
+
+		return;
+	}
+
+	network->autoconnect = value;
+}
+
+static const char *get_autoconnect_tostr(const void *data)
+{
+	const struct known_network *network = data;
+
+	return network->autoconnect ? "yes" : "no";
+}
+
 static const struct proxy_interface_property known_network_properties[] = {
-	{ "Name",              "s", update_name           },
+	{ "Name",              "s", update_name, get_name },
 	{ "Type",              "s", update_type           },
 	{ "LastConnectedTime", "s", update_last_connected },
 	{ "Hidden",            "b", update_hidden, get_hidden_tostr },
+	{ "Autoconnect",       "b", update_autoconnect, get_autoconnect_tostr,
+		true, properties_builder_append_yes_no_variant,
+		properties_yes_no_opts },
 	{ },
 };
 
@@ -156,6 +189,20 @@ static void known_network_destroy(void *data)
 	l_free(network);
 }
 
+static void known_network_display(const struct proxy_interface *proxy)
+{
+	const struct known_network *known_network =
+						proxy_interface_get_data(proxy);
+	char *caption = l_strdup_printf("%s: %s", "Known Network",
+							known_network->name);
+
+	proxy_properties_display(proxy, caption, MARGIN, 18, 50);
+
+	l_free(caption);
+
+	display_table_footer();
+}
+
 static void known_network_display_inline(const char *margin, const void *data)
 {
 	const struct known_network *network = data;
@@ -227,17 +274,36 @@ static enum cmd_status cmd_list(const char *entity, char **args, int argc)
 	return CMD_STATUS_DONE;
 }
 
-static enum cmd_status cmd_forget(const char *entity, char **argv, int argc)
+static const struct proxy_interface *known_network_proxy_find_by_name(
+							const char *name)
 {
 	struct network_args network_args;
 	struct l_queue *match;
-	const struct proxy_interface *known_network_proxy;
+	const struct proxy_interface *proxy;
 
-	if (argc < 1)
-		return CMD_STATUS_INVALID_ARGS;
+	if (!name)
+		return NULL;
 
-	network_args.name = argv[0];
-	network_args.type = argc >= 2 ? argv[1] : NULL;
+	if (l_str_has_suffix(name, ".psk"))
+		network_args.type = "psk";
+	else if (l_str_has_suffix(name, ".8021x"))
+		network_args.type = "8021x";
+	else if (l_str_has_suffix(name, ".open"))
+		network_args.type = "open";
+	else
+		network_args.type = NULL;
+
+	if (network_args.type) {
+		char *dot = strrchr(name, '.');
+
+		if (!dot)
+			/* This shouldn't ever be the case */
+			return NULL;
+
+		*dot = '\0';
+	}
+
+	network_args.name = name;
 
 	match = proxy_interface_find_all(known_network_interface_type.interface,
 						known_network_match,
@@ -245,37 +311,109 @@ static enum cmd_status cmd_forget(const char *entity, char **argv, int argc)
 
 	if (!match) {
 		display("No network with specified parameters was found\n");
-		return CMD_STATUS_INVALID_VALUE;
+		return NULL;
 	}
 
 	if (l_queue_length(match) > 1) {
 		if (!network_args.type) {
 			display("Provided network name is ambiguous. "
-				"Please specify security type.\n");
+				"Specify network security type as follows:\n");
+			display("<\"network name" COLOR_BOLDGRAY ".security"
+							COLOR_OFF "\">\n");
+			display("\twhere '.security' is [.psk | .8021x | "
+								".open]\n");
 		}
 
 		l_queue_destroy(match, NULL);
-		return CMD_STATUS_INVALID_VALUE;
+		return NULL;
 	}
 
-	known_network_proxy = l_queue_pop_head(match);
+	proxy = l_queue_pop_head(match);
 	l_queue_destroy(match, NULL);
 
-	proxy_interface_method_call(known_network_proxy, "Forget", "",
-					check_errors_method_callback);
+	return proxy;
+}
+
+static enum cmd_status cmd_forget(const char *network_name, char **argv,
+								int argc)
+{
+	const struct proxy_interface *proxy =
+				known_network_proxy_find_by_name(network_name);
+
+	if (!proxy)
+		return CMD_STATUS_INVALID_ARGS;
+
+	proxy_interface_method_call(proxy, "Forget", "",
+						check_errors_method_callback);
 
 	return CMD_STATUS_TRIGGERED;
 }
 
+static enum cmd_status cmd_show(const char *network_name, char **argv, int argc)
+{
+	const struct proxy_interface *proxy =
+				known_network_proxy_find_by_name(network_name);
+
+	if (!proxy)
+		return CMD_STATUS_INVALID_ARGS;
+
+	known_network_display(proxy);
+
+	return CMD_STATUS_DONE;
+}
+
+static void property_set_callback(struct l_dbus_message *message,
+								void *user_data)
+{
+	dbus_message_has_error(message);
+}
+
+static enum cmd_status cmd_set_property(const char *network_name,
+							char **argv, int argc)
+{
+	const struct proxy_interface *proxy =
+				known_network_proxy_find_by_name(network_name);
+
+	if (!proxy)
+		return CMD_STATUS_INVALID_VALUE;
+
+	if (argc != 2)
+		return CMD_STATUS_INVALID_ARGS;
+
+	if (!proxy_property_set(proxy, argv[0], argv[1],
+						property_set_callback))
+		return CMD_STATUS_INVALID_VALUE;
+
+	return CMD_STATUS_TRIGGERED;
+}
+
+static char *set_property_cmd_arg_completion(const char *text, int state)
+{
+	return proxy_property_completion(known_network_properties, text, state);
+}
+
+static bool match_by_partial_name(const void *a, const void *b)
+{
+	const struct known_network *network = a;
+	const char *text = b;
+
+	return !strncmp(network->name, text, strlen(text));
+}
+
 static const struct command known_networks_commands[] = {
 	{ NULL, "list",   NULL, cmd_list,   "List known networks", true },
-	{ NULL, "forget", "<\"network name\"> [security]",
-				cmd_forget, "Forget known network" },
+	{ "<\"network name\">", "forget", NULL, cmd_forget,
+						"Forget known network" },
+	{ "<\"network name\">", "show", NULL, cmd_show, "Show known network",
+		true },
+	{ "<\"network name\">", "set-property", "<name> <value>",
+		cmd_set_property, "Set property", false,
+		set_property_cmd_arg_completion },
 	{ }
 };
-
 static char *family_arg_completion(const char *text, int state)
 {
+	static bool first_pass;
 	static size_t index;
 	static size_t len;
 	const char *cmd;
@@ -283,19 +421,31 @@ static char *family_arg_completion(const char *text, int state)
 	if (!state) {
 		index = 0;
 		len = strlen(text);
+		first_pass = true;
 	}
 
 	while ((cmd = known_networks_commands[index].cmd)) {
 		if (known_networks_commands[index++].entity)
 			continue;
 
-		if (strncmp(cmd, text, len))
-			continue;
-
-		return l_strdup(cmd);
+		if (!strncmp(cmd, text, len))
+			return l_strdup(cmd);
 	}
 
-	return NULL;
+	if (first_pass) {
+		state = 0;
+		first_pass = false;
+	}
+
+	return proxy_property_str_completion(&known_network_interface_type,
+						match_by_partial_name, "Name",
+						text, state, NULL);
+}
+
+static char *entity_arg_completion(const char *text, int state)
+{
+	return command_entity_arg_completion(text, state,
+						known_networks_commands);
 }
 
 static struct command_family known_networks_command_family = {
@@ -303,6 +453,7 @@ static struct command_family known_networks_command_family = {
 	.name = "known-networks",
 	.command_list = known_networks_commands,
 	.family_arg_completion = family_arg_completion,
+	.entity_arg_completion = entity_arg_completion,
 };
 
 static int known_networks_command_family_init(void)
