@@ -217,12 +217,21 @@ enum ie_type {
 	/* Reserved 207 - 220 */
 	IE_TYPE_VENDOR_SPECIFIC                      = 221,
 	/* Reserved 222 - 254 */
+	IE_TYPE_FILS_INDICATION                      = 240,
 	IE_TYPE_EXTENSION                            = 255,
 
-	/* Reserved extensions 0 - 8 */
+	IE_TYPE_FILS_REQUEST_PARAMETERS              = 256 + 2,
+	IE_TYPE_FILS_KEY_CONFIRMATION                = 256 + 3,
+	IE_TYPE_FILS_SESSION                         = 256 + 4,
+	IE_TYPE_FILS_HLP_CONTAINER                   = 256 + 5,
+	IE_TYPE_FILS_IP_ADDRESS                      = 256 + 6,
+	IE_TYPE_KEY_DELIVERY                         = 256 + 7,
+	IE_TYPE_FILS_WRAPPED_DATA                    = 256 + 8,
 	IE_TYPE_FTM_SYNCHRONIZATION_INFORMATION      = 256 + 9,
 	IE_TYPE_EXTENDED_REQUEST                     = 256 + 10,
 	IE_TYPE_ESTIMATED_SERVICE_PARAMETERS         = 256 + 11,
+	IE_TYPE_FILS_PUBLIC_KEY                      = 256 + 12,
+	IE_TYPE_FILS_NONCE                           = 256 + 13,
 	IE_TYPE_FUTURE_CHANNEL_GUIDANCE              = 256 + 14,
 	IE_TYPE_OWE_DH_PARAM                         = 256 + 32,
 };
@@ -252,12 +261,19 @@ enum ie_rsn_akm_suite {
 	IE_RSN_AKM_SUITE_8021X_SUITE_B_SHA384	= 0x0800,
 	IE_RSN_AKM_SUITE_FT_OVER_8021X_SHA384	= 0x1000,
 	IE_RSN_AKM_SUITE_OWE			= 0x2000,
+	IE_RSN_AKM_SUITE_FILS_SHA256		= 0x4000,
+	IE_RSN_AKM_SUITE_FILS_SHA384		= 0x8000,
+	IE_RSN_AKM_SUITE_FT_OVER_FILS_SHA256	= 0x10000,
+	IE_RSN_AKM_SUITE_FT_OVER_FILS_SHA384	= 0x20000,
 };
 
 #define IE_AKM_IS_SAE(akm) \
 	((akm == IE_RSN_AKM_SUITE_SAE_SHA256) || \
 	(akm == IE_RSN_AKM_SUITE_FT_OVER_SAE_SHA256))
 
+#define IE_LEN(ie) \
+	(ie) ? (ie)[1] + 2 : 0
+
 struct ie_tlv_iter {
 	unsigned int max;
 	unsigned int pos;
@@ -284,7 +300,7 @@ struct ie_tlv_builder {
 struct ie_rsn_info {
 	enum ie_rsn_cipher_suite group_cipher;
 	uint16_t pairwise_ciphers;
-	uint16_t akm_suites;
+	uint32_t akm_suites;
 	bool preauthentication:1;
 	bool no_pairwise:1;
 	uint8_t ptksa_replay_counter:2;
@@ -410,10 +426,18 @@ int ie_parse_bss_load_from_data(const uint8_t *data, uint8_t len,
 				uint8_t *out_channel_utilization,
 				uint16_t *out_admission_capacity);
 
-int ie_parse_supported_rates(struct ie_tlv_iter *iter,
-				struct l_uintset **set);
-int ie_parse_supported_rates_from_data(const uint8_t *data, uint8_t len,
-				struct l_uintset **set);
+int ie_parse_supported_rates_from_data(const uint8_t *supp_rates_ie,
+					uint8_t supp_rates_len,
+					const uint8_t *ext_supp_rates_ie,
+					uint8_t ext_supp_rates_len,
+					int32_t rssi, uint64_t *data_rate);
+
+int ie_parse_data_rates(const uint8_t *supp_rates_ie,
+			const uint8_t *ext_supp_rates_ie,
+			const uint8_t *ht_ie,
+			const uint8_t *vht_ie,
+			int32_t rssi,
+			uint64_t *data_rate);
 
 int ie_parse_mobility_domain(struct ie_tlv_iter *iter, uint16_t *mdid,
 				bool *ft_over_ds, bool *resource_req);
