@@ -103,6 +103,18 @@ static void known_network_register_dbus(struct network_info *network)
 						L_DBUS_INTERFACE_PROPERTIES);
 }
 
+static void known_network_set_autoconnect(struct network_info *network,
+							bool autoconnect)
+{
+	if (network->is_autoconnectable == autoconnect)
+		return;
+
+	network->is_autoconnectable = autoconnect;
+
+	l_dbus_property_changed(dbus_get_bus(), known_network_get_path(network),
+				IWD_KNOWN_NETWORK_INTERFACE, "Autoconnect");
+}
+
 static void known_network_update(struct network_info *orig_network,
 					const char *ssid,
 					enum security security,
@@ -111,6 +123,7 @@ static void known_network_update(struct network_info *orig_network,
 {
 	struct network_info *network;
 	bool is_hidden = false;
+	bool is_autoconnectable;
 
 	if (orig_network)
 		network = orig_network;
@@ -148,6 +161,13 @@ static void known_network_update(struct network_info *orig_network,
 	if (network->is_hidden)
 		num_known_hidden_networks++;
 
+	if (!l_settings_get_bool(settings, "Settings", "Autoconnect",
+							&is_autoconnectable))
+		/* If no entry, default to Autoconnectable=True */
+		is_autoconnectable = true;
+
+	known_network_set_autoconnect(network, is_autoconnectable);
+
 	if (orig_network)
 		return;
 
@@ -239,6 +259,48 @@ static bool known_network_property_get_hidden(struct l_dbus *dbus,
 	return true;
 }
 
+static bool known_network_property_get_autoconnect(struct l_dbus *dbus,
+					struct l_dbus_message *message,
+					struct l_dbus_message_builder *builder,
+					void *user_data)
+{
+	struct network_info *network = user_data;
+	bool autoconnect = network->is_autoconnectable;
+
+	l_dbus_message_builder_append_basic(builder, 'b', &autoconnect);
+
+	return true;
+}
+
+static struct l_dbus_message *known_network_property_set_autoconnect(
+					struct l_dbus *dbus,
+					struct l_dbus_message *message,
+					struct l_dbus_message_iter *new_value,
+					l_dbus_property_complete_cb_t complete,
+					void *user_data)
+{
+	struct network_info *network = user_data;
+	struct l_settings *settings;
+	bool autoconnect;
+
+	if (!l_dbus_message_iter_get_variant(new_value, "b", &autoconnect))
+		return dbus_error_invalid_args(message);
+
+	if (network->is_autoconnectable == autoconnect)
+		return l_dbus_message_new_method_return(message);
+
+	settings = storage_network_open(network->type, network->ssid);
+	if (!settings)
+		return dbus_error_failed(message);
+
+	l_settings_set_bool(settings, "Settings", "Autoconnect", autoconnect);
+
+	storage_network_sync(network->type, network->ssid, settings);
+	l_settings_free(settings);
+
+	return l_dbus_message_new_method_return(message);
+}
+
 static bool known_network_property_get_last_connected(struct l_dbus *dbus,
 					struct l_dbus_message *message,
 					struct l_dbus_message_builder *builder,
@@ -273,6 +335,9 @@ static void setup_known_network_interface(struct l_dbus_interface *interface)
 	l_dbus_interface_property(interface, "Hidden", 0, "b",
 					known_network_property_get_hidden,
 					NULL);
+	l_dbus_interface_property(interface, "Autoconnect", 0, "b",
+					known_network_property_get_autoconnect,
+					known_network_property_set_autoconnect);
 	l_dbus_interface_property(interface, "LastConnectedTime", 0, "s",
 				known_network_property_get_last_connected,
 				NULL);
