@@ -44,6 +44,8 @@
 #include "src/plugin.h"
 #include "src/simauth.h"
 #include "src/adhoc.h"
+#include "src/blacklist.h"
+#include "src/storage.h"
 
 #include "src/backtrace.h"
 
@@ -149,6 +151,8 @@ static void nl80211_appeared(void *user_data)
 
 	l_debug("Found nl80211 interface");
 
+	manager_init(nl80211);
+
 	if (!wiphy_init(nl80211, phys, nophys))
 		l_error("Unable to init wiphy functionality");
 
@@ -165,6 +169,7 @@ static void nl80211_vanished(void *user_data)
 {
 	l_debug("Lost nl80211 interface");
 
+	manager_exit();
 	ap_exit();
 	adhoc_exit();
 	scan_exit();
@@ -469,6 +474,11 @@ int main(int argc, char *argv[])
 
 	exit_status = EXIT_FAILURE;
 
+	if (create_dirs(DAEMON_STORAGEDIR "/")) {
+		l_error("Failed to create " DAEMON_STORAGEDIR "/");
+		goto fail_dbus;
+	}
+
 	dbus = l_dbus_new_default(L_DBUS_SYSTEM_BUS);
 	if (!dbus) {
 		l_error("Failed to initialize D-Bus");
@@ -498,9 +508,11 @@ int main(int argc, char *argv[])
 	known_networks_init();
 	sim_auth_init();
 	plugin_init(plugins, noplugins);
+	blacklist_init();
 
 	exit_status = l_main_run_with_signal(signal_handler, NULL);
 
+	blacklist_exit();
 	plugin_exit();
 	sim_auth_exit();
 	known_networks_exit();
