@@ -62,15 +62,44 @@ enum netdev_iftype {
 	NETDEV_IFTYPE_ADHOC = 1,
 	NETDEV_IFTYPE_STATION = 2,
 	NETDEV_IFTYPE_AP = 3,
+	NETDEV_IFTYPE_P2P_CLIENT,
+	NETDEV_IFTYPE_P2P_GO,
 };
 
 typedef void (*netdev_command_cb_t)(struct netdev *netdev, int result,
 						void *user_data);
+/*
+ * Callback for a connection attempt. This callback is called on both success
+ * and failure. Depending on result, the event_data will have different
+ * meanings:
+ *
+ * NETDEV_RESULT_OK - unused
+ * NETDEV_RESULT_AUTHENTICATION_FAILED - MMPDU_STATUS_CODE
+ * NETDEV_RESULT_ASSOCIATION_FAILED - MMPDU_STATUS_CODE
+ * NETDEV_RESULT_HANDSHAKE_FAILED - MMPDU_REASON_CODE
+ * NETDEV_RESULT_KEY_SETTINGS_FAILED - unused
+ * NETDEV_RESULT_ABORTED - unused.
+ */
 typedef void (*netdev_connect_cb_t)(struct netdev *netdev,
 					enum netdev_result result,
+					void *event_data,
 					void *user_data);
+/*
+ * Notify function for netdev events. Depending on the event type, event_data
+ * will have different meanings:
+ *
+ * NETDEV_EVENT_AUTHENTICATING - unsused
+ * NETDEV_EVENT_ASSOCIATING - unused
+ * NETDEV_EVENT_LOST_BEACON - unused
+ * NETDEV_EVENT_DISCONNECT_BY_AP - MMPDU_REASON_CODE
+ * NETDEV_EVENT_DISCONNECT_BY_SME - MMPDU_REASON_CODE
+ * NETDEV_EVENT_RSSI_THRESHOLD_LOW - unused
+ * NETDEV_EVENT_RSSI_THRESHOLD_HIGH - unused
+ * NETDEV_EVENT_RSSI_LEVEL_NOTIFY - rssi level index (uint8_t)
+ */
 typedef void (*netdev_event_func_t)(struct netdev *netdev,
 					enum netdev_event event,
+					void *event_data,
 					void *user_data);
 typedef void (*netdev_disconnect_cb_t)(struct netdev *netdev, bool result,
 					void *user_data);
@@ -155,7 +184,6 @@ int netdev_neighbor_report_req(struct netdev *netdev,
 
 int netdev_set_rssi_report_levels(struct netdev *netdev, const int8_t *levels,
 					size_t levels_num);
-int netdev_get_rssi_level(struct netdev *netdev);
 
 uint32_t netdev_frame_watch_add(struct netdev *netdev, uint16_t frame_type,
 				const uint8_t *prefix, size_t prefix_len,
@@ -175,3 +203,6 @@ uint32_t netdev_station_watch_add(struct netdev *netdev,
 		netdev_station_watch_func_t func, void *user_data);
 
 bool netdev_station_watch_remove(struct netdev *netdev, uint32_t id);
+
+struct netdev *netdev_create_from_genl(struct l_genl_msg *msg);
+bool netdev_destroy(struct netdev *netdev);
