@@ -41,11 +41,21 @@ enum ie_rsn_cipher_suite wiphy_select_cipher(struct wiphy *wiphy,
 enum ie_rsn_akm_suite wiphy_select_akm(struct wiphy *wiphy,
 					struct scan_bss *bss);
 
+bool wiphy_parse_id_and_name(struct l_genl_attr *attr, uint32_t *out_id,
+				const char **out_name);
+
 struct wiphy *wiphy_find(int wiphy_id);
 
+struct wiphy *wiphy_create(uint32_t wiphy_id, const char *name);
+bool wiphy_destroy(struct wiphy *wiphy);
+void wiphy_update_from_genl(struct wiphy *wiphy, struct l_genl_msg *msg);
+
 const char *wiphy_get_path(struct wiphy *wiphy);
 uint32_t wiphy_get_supported_bands(struct wiphy *wiphy);
+const struct scan_freq_set *wiphy_get_supported_freqs(
+						const struct wiphy *wiphy);
 bool wiphy_can_connect(struct wiphy *wiphy, struct scan_bss *bss);
+bool wiphy_can_randomize_mac_addr(struct wiphy *wiphy);
 bool wiphy_has_feature(struct wiphy *wiphy, uint32_t feature);
 bool wiphy_has_ext_feature(struct wiphy *wiphy, uint32_t feature);
 uint8_t wiphy_get_max_num_ssids_per_scan(struct wiphy *wiphy);
